@@ -5,7 +5,7 @@
 #
 ###
 #
-#  Copyright (c) 1994 - 97	David Albert Bagley, bagleyd@bigfoot.com
+#  Copyright (c) 1994 - 99	David Albert Bagley, bagleyd@tux.org
 #
 #                   All Rights Reserved
 #
@@ -50,11 +50,11 @@ typedef struct _RTT {
 	int         row, trbl, tlbr;
 } RTT;
 
-static void InitializeOct(Widget request, Widget new);
-static void ExposeOct(Widget new, XEvent * event, Region region);
+static void InitializeOct(Widget request, Widget renew);
+static void ExposeOct(Widget renew, XEvent * event, Region region);
 static void ResizeOct(OctWidget w);
 static void DestroyOct(Widget old);
-static Boolean SetValuesOct(Widget current, Widget request, Widget new);
+static Boolean SetValuesOct(Widget current, Widget request, Widget renew);
 static void QuitOct(OctWidget w, XEvent * event, char **args, int nArgs);
 static void PracticeOct(OctWidget w, XEvent * event, char **args, int nArgs);
 static void PracticeOctMaybe(OctWidget w, XEvent * event, char **args, int nArgs);
@@ -84,14 +84,16 @@ static void MoveOctCcw(OctWidget w, XEvent * event, char **args, int nArgs);
 static void MoveOctInput(OctWidget w, int x, int y, int direction, int shift, int control);
 static void SelectOct(OctWidget w, XEvent * event, char **args, int nArgs);
 static void ReleaseOct(OctWidget w, XEvent * event, char **args, int nArgs);
-static void GetColor(OctWidget w, int face, int init);
+
+static void SetAllColors(OctWidget w, Boolean init);
+static void GetColor(OctWidget w, int face, Boolean init);
 static void MoveControlCb(OctWidget w, int face, int direction, int style);
 static void CheckPolyhedrons(OctWidget w);
 static void ResetPolyhedrons(OctWidget w);
 static void ResizePolyhedrons(OctWidget w);
-static int  SelectPolyhedrons(OctWidget w, int x, int y, int *face, RTT * rtt);
-static int  NarrowSelection(OctWidget w, int style, int *face, int *direction);
-static int  PositionPolyhedrons(OctWidget w, int x, int y, int style, int *face, RTT * rtt, int *direction);
+static Boolean SelectPolyhedrons(OctWidget w, int x, int y, int *face, RTT * rtt);
+static Boolean NarrowSelection(OctWidget w, int style, int *face, int *direction);
+static Boolean PositionPolyhedrons(OctWidget w, int x, int y, int style, int *face, RTT * rtt, int *direction);
 static void MoveNoPolyhedrons(OctWidget w);
 static void PracticePolyhedrons(OctWidget w);
 static void RandomizePolyhedrons(OctWidget w);
@@ -105,7 +107,7 @@ static void ReverseRTT(OctWidget w, int len, int orient);
 static void WriteRTT(OctWidget w, int view, int side, int dir, int h, int len, int orient);
 static void DrawFrame(OctWidget w, GC gc);
 static void DrawTriangle(OctWidget w, int face, int position, int offset);
-static void DrawOrientLine(OctWidget w, int orient, int face, int side, int dx, int dy);
+static void DrawOrientLine(OctWidget w, int orient, int face, int side, int dx, int dy, GC borderGC);
 static int  Length(OctWidget w, int dir, int h);
 static int  CheckMoveDir(OctWidget w, RTT rtt1, RTT rtt2, int face, int *direction);
 static RTT  ToRTT(int position);
@@ -233,6 +235,8 @@ static XtResource resourcesOct[] =
 	 XtOffset(OctWidget, core.height), XtRString, "400"},
 	{XtNmono, XtCMono, XtRBoolean, sizeof (Boolean),
 	 XtOffset(OctWidget, oct.mono), XtRString, "FALSE"},
+	{XtNreverse, XtCReverse, XtRBoolean, sizeof (Boolean),
+	 XtOffset(OctWidget, oct.reverse), XtRString, "FALSE"},
 	{XtNsize, XtCSize, XtRInt, sizeof (int),
 	 XtOffset(OctWidget, oct.size), XtRString, "3"},	/*DEFAULTOCTAS */
 	{XtNsticky, XtCSticky, XtRBoolean, sizeof (Boolean),
@@ -461,11 +465,9 @@ static XPoint triangleUnit[MAXSIDES][4] =
 static XPoint triangleList[MAXSIDES][4], letterList[MAXSIDES];
 
 static void
-InitializeOct(Widget request, Widget new)
+InitializeOct(Widget request, Widget renew)
 {
-	OctWidget   w = (OctWidget) new;
-	XGCValues   values;
-	XtGCMask    valueMask;
+	OctWidget   w = (OctWidget) renew;
 	int         face, orient;
 
 	for (face = 0; face < MAXFACES; face++)
@@ -477,19 +479,8 @@ InitializeOct(Widget request, Widget new)
 	InitMoves();
 	ResetPolyhedrons(w);
 	(void) SRAND(getpid());
-	valueMask = GCForeground | GCBackground;
-	values.background = w->core.background_pixel;
-	values.foreground = w->oct.foreground;
-	w->oct.puzzleGC = XtGetGC(new, valueMask, &values);
-	values.foreground = w->oct.borderColor;
-	w->oct.borderGC = XtGetGC(new, valueMask, &values);
 	w->oct.depth = DefaultDepthOfScreen(XtScreen(w));
-	valueMask = GCForeground | GCBackground;
-	values.foreground = w->core.background_pixel;
-	values.background = w->oct.foreground;
-	w->oct.inverseGC = XtGetGC(new, valueMask, &values);
-	for (face = 0; face < MAXFACES; face++)
-		GetColor(w, face, TRUE);
+	SetAllColors(w, True);
 	ResizeOct(w);
 }
 
@@ -535,82 +526,59 @@ ResizeOct(OctWidget w)
 }
 
 static void
-ExposeOct(Widget new, XEvent * event, Region region)
-		 /* Not used */
+ExposeOct(Widget renew, XEvent * event, Region region)
 {
-	OctWidget   w = (OctWidget) new;
+	OctWidget   w = (OctWidget) renew;
 
 	if (w->core.visible) {
+		if (w->oct.reverse)
+			XFillRectangle(XtDisplay(w), XtWindow(w),
+				       w->oct.inverseGC, 0, 0, w->core.width, w->core.height);
 		DrawFrame(w, w->oct.puzzleGC);
 		DrawAllPolyhedrons(w);
 	}
 }
 
 static      Boolean
-SetValuesOct(Widget current, Widget request, Widget new)
+SetValuesOct(Widget current, Widget request, Widget renew)
 {
-	OctWidget   c = (OctWidget) current, w = (OctWidget) new;
-	XGCValues   values;
-	XtGCMask    valueMask;
-	Boolean     redraw = FALSE;
+	OctWidget   c = (OctWidget) current, w = (OctWidget) renew;
+	Boolean     redraw = False, setColors = False;
 	int         face;
 
 	CheckPolyhedrons(w);
-	if (w->oct.foreground != c->oct.foreground) {
-		valueMask = GCForeground | GCBackground;
-		values.foreground = w->oct.foreground;
-		values.background = w->core.background_pixel;
-		XtReleaseGC(new, w->oct.puzzleGC);
-		w->oct.puzzleGC = XtGetGC(new, valueMask, &values);
-		redraw = TRUE;
-	}
-	if (w->core.background_pixel != c->core.background_pixel) {
-		valueMask = GCForeground | GCBackground;
-		values.foreground = w->core.background_pixel;
-		values.background = w->oct.foreground;
-		XtReleaseGC(new, w->oct.inverseGC);
-		w->oct.inverseGC = XtGetGC(new, valueMask, &values);
-		redraw = TRUE;
-	}
-	if (w->oct.borderColor != c->oct.borderColor) {
-		valueMask = GCForeground | GCBackground;
-		values.foreground = w->oct.borderColor;
-		values.background = w->core.background_pixel;
-		XtReleaseGC(new, w->oct.borderGC);
-		w->oct.borderGC = XtGetGC(new, valueMask, &values);
-		redraw = TRUE;
-	}
-	if (w->oct.mono || w->oct.depth == 1) {
-		valueMask = GCForeground | GCBackground;
-		values.background = w->core.background_pixel;
-		values.foreground = w->oct.foreground;
-		for (face = 0; face < MAXFACES; face++) {
-			XtReleaseGC(new, w->oct.faceGC[face]);
-			w->oct.faceGC[face] = XtGetGC(new, valueMask, &values);
-		}
-		redraw = TRUE;
-	}
 	for (face = 0; face < MAXFACES; face++) {
-		if (strcmp(w->oct.faceName[face], c->oct.faceName[face]))
-			GetColor(w, face, FALSE);
+		if (strcmp(w->oct.faceName[face], c->oct.faceName[face])) {
+			setColors = True;
+			break;
+		}
+	}
+	if (w->core.background_pixel != c->core.background_pixel ||
+	    w->oct.foreground != c->oct.foreground ||
+	    w->oct.borderColor != c->oct.borderColor ||
+	    w->oct.reverse != c->oct.reverse ||
+	    w->oct.mono != c->oct.mono ||
+	    setColors) {
+		SetAllColors(w, False);
+		redraw = True;
 	}
 	if (w->oct.orient != c->oct.orient) {
 		ResetPolyhedrons(w);
-		redraw = TRUE;
+		redraw = True;
 	} else if (w->oct.practice != c->oct.practice) {
 		ResetPolyhedrons(w);
-		redraw = TRUE;
+		redraw = True;
 	}
 	if (w->oct.size != c->oct.size ||
 	    w->oct.mode != c->oct.mode ||
 	    w->oct.sticky != c->oct.sticky) {
 		ResetPolyhedrons(w);
 		ResizeOct(w);
-		redraw = TRUE;
+		redraw = True;
 	}
 	if (w->oct.octaLength != c->oct.octaLength) {
 		ResizeOct(w);
-		redraw = TRUE;
+		redraw = True;
 	}
 	return (redraw);
 }
@@ -631,8 +599,8 @@ SelectOct(OctWidget w, XEvent * event, char **args, int nArgs)
 	if (SelectPolyhedrons(w, event->xbutton.x, event->xbutton.y,
 			      &(w->oct.currentFace), &rtt)) {
 		control = (int) (event->xkey.state & ControlMask);
+		w->oct.currentPosition = ToPosition(rtt);
 		if (control || w->oct.practice || !CheckSolved(w)) {
-			w->oct.currentPosition = ToPosition(rtt);
 			DrawTriangle(w, w->oct.currentFace, w->oct.currentPosition, TRUE);
 		}
 	} else
@@ -827,7 +795,9 @@ UndoOct(OctWidget w, XEvent * event, char **args, int nArgs)
 static void
 SolveOct(OctWidget w, XEvent * event, char **args, int nArgs)
 {
-	/* SolvePolyhedrons(w); *//* Sorry, unimplemented */
+#if 0
+	SolvePolyhedrons(w);	/* Sorry, this is not implemented */
+#endif
 }
 
 static void
@@ -1020,14 +990,66 @@ MoveOct(OctWidget w, int face, int position, int direction, int style, int contr
 }
 
 static void
-GetColor(OctWidget w, int face, int init)
+SetAllColors(OctWidget w, Boolean init)
+{
+	XGCValues   values;
+	XtGCMask    valueMask;
+	int         face;
+
+	valueMask = GCForeground | GCBackground;
+
+	if (w->oct.reverse) {
+		values.background = w->core.background_pixel;
+		values.foreground = w->oct.foreground;
+	} else {
+		values.foreground = w->core.background_pixel;
+		values.background = w->oct.foreground;
+	}
+	if (!init)
+		XtReleaseGC((Widget) w, w->oct.inverseGC);
+	w->oct.inverseGC = XtGetGC((Widget) w, valueMask, &values);
+	if (w->oct.reverse) {
+		values.background = w->oct.foreground;
+		values.foreground = w->core.background_pixel;
+	} else {
+		values.foreground = w->oct.foreground;
+		values.background = w->core.background_pixel;
+	}
+	if (!init)
+		XtReleaseGC((Widget) w, w->oct.puzzleGC);
+	w->oct.puzzleGC = XtGetGC((Widget) w, valueMask, &values);
+	if (w->oct.depth < 2 || w->oct.mono) {
+		if (w->oct.reverse) {
+			values.background = w->oct.foreground;
+			values.foreground = w->core.background_pixel;
+		} else {
+			values.foreground = w->oct.foreground;
+			values.background = w->core.background_pixel;
+		}
+	} else {
+		values.foreground = w->oct.borderColor;
+		values.background = w->core.background_pixel;
+	}
+	if (!init)
+		XtReleaseGC((Widget) w, w->oct.borderGC);
+	w->oct.borderGC = XtGetGC((Widget) w, valueMask, &values);
+	for (face = 0; face < MAXFACES; face++)
+		GetColor(w, face, init);
+}
+
+static void
+GetColor(OctWidget w, int face, Boolean init)
 {
 	XGCValues   values;
 	XtGCMask    valueMask;
 	XColor      colorCell, rgb;
 
 	valueMask = GCForeground | GCBackground;
-	values.background = w->core.background_pixel;
+	if (w->oct.reverse) {
+		values.background = w->oct.foreground;
+	} else {
+		values.background = w->core.background_pixel;
+	}
 	if (w->oct.depth > 1 && !w->oct.mono) {
 		if (XAllocNamedColor(XtDisplay(w),
 				  DefaultColormap(XtDisplay(w), XtWindow(w)),
@@ -1045,7 +1067,13 @@ GetColor(OctWidget w, int face, int init)
 			XtWarning(buf);
 		}
 	}
-	values.foreground = w->oct.foreground;
+	if (w->oct.reverse) {
+		values.background = w->oct.foreground;
+		values.foreground = w->core.background_pixel;
+	} else {
+		values.background = w->core.background_pixel;
+		values.foreground = w->oct.foreground;
+	}
 	if (!init)
 		XtReleaseGC((Widget) w, w->oct.faceGC[face]);
 	w->oct.faceGC[face] = XtGetGC((Widget) w, valueMask, &values);
@@ -1202,7 +1230,7 @@ ResetPolyhedrons(OctWidget w)
 			w->oct.octaLoc[face][position].rotation = 3 * face % MAXORIENT;
 		}
 	FlushMoves(w);
-	w->oct.started = FALSE;
+	w->oct.started = False;
 }
 
 static void
@@ -1228,7 +1256,7 @@ ResizePolyhedrons(OctWidget w)
 	w->oct.orientLineLength = w->oct.octaLength / 4;
 }
 
-static int
+static      Boolean
 SelectPolyhedrons(OctWidget w, int x, int y, int *face, RTT * rtt)
 {
 	int         view;
@@ -1246,21 +1274,21 @@ SelectPolyhedrons(OctWidget w, int x, int y, int *face, RTT * rtt)
 	if (x <= 0 || y <= 0 ||
 	    x >= w->oct.faceLength + w->oct.delta ||
 	    y >= w->oct.faceLength + w->oct.delta)
-		return FALSE;
+		return False;
 	else if (x + y > w->oct.faceLength) {
 		if (x > y)
 			*face = 1;
 		else if (x < y)
 			*face = 2;
 		else
-			return FALSE;
+			return False;
 	} else {
 		if (x > y)
 			*face = 0;
 		else if (x < y)
 			*face = 3;
 		else
-			return FALSE;
+			return False;
 	}
 	rtt->row = 0;
 	while ((x <= (w->oct.size - (rtt->row + 1)) *
@@ -1293,10 +1321,10 @@ SelectPolyhedrons(OctWidget w, int x, int y, int *face, RTT * rtt)
 		rtt->tlbr = rtt->trbl;
 		rtt->trbl = view;
 	}
-	return TRUE;
+	return True;
 }
 
-static int
+static      Boolean
 NarrowSelection(OctWidget w, int style, int *face, int *direction)
 {
 	int         side, view;
@@ -1306,7 +1334,7 @@ NarrowSelection(OctWidget w, int style, int *face, int *direction)
 	if (!w->oct.vertical && view == DOWN)
 		*direction = (*direction + MAXSIDES) % MAXFACES;
 	if (!(*direction % 2) && !((*direction / 2 + side) % 2))
-		return FALSE;
+		return False;
 	if (style == PERIOD4) {
 		if (!(*direction % 2)) {
 			if (*direction == (2 * side + 2) % MAXFACES)
@@ -1318,14 +1346,14 @@ NarrowSelection(OctWidget w, int style, int *face, int *direction)
 		   rtt->trbl; else *//* (*direction / 2) % 2 == 1 *//*
 		   h = rtt->tlbr; */
 	}
-	return TRUE;
+	return True;
 }
 
-static int
+static      Boolean
 PositionPolyhedrons(OctWidget w, int x, int y, int style, int *face, RTT * rtt, int *direction)
 {
 	if (!SelectPolyhedrons(w, x, y, face, rtt))
-		return FALSE;
+		return False;
 	return NarrowSelection(w, style, face, direction);
 }
 
@@ -1403,16 +1431,17 @@ RandomizePolyhedrons(OctWidget w)
 			if (style == PERIOD3) {
 				randomDirection = NRAND(MAXORIENT / 2);
 				if (randomDirection >= MAXSIDES) {
-					if (randomDirection == 4)
+					if (randomDirection == 4) {
 						if (face % 2)
 							randomDirection = BOTTOM;
 						else
 							randomDirection = RIGHT;
-					else if (randomDirection == 5)
+					} else if (randomDirection == 5) {
 						if (face % 2)
 							randomDirection = TOP;
 						else
 							randomDirection = LEFT;
+					}
 				} else
 					randomDirection = randomDirection * 2 + 1;;
 			} else {	/* style == PERIOD4 */
@@ -1793,6 +1822,7 @@ DrawAllPolyhedrons(OctWidget w)
 static void
 DrawTriangle(OctWidget w, int face, int position, int offset)
 {
+	GC          faceGC, borderGC;
 	int         dx = 0, dy = 0;
 	int         side, row = Sqrt(position);
 	int         base = row * row;
@@ -1859,20 +1889,21 @@ DrawTriangle(OctWidget w, int face, int position, int offset)
 	triangleList[side][0].x = dx;
 	triangleList[side][0].y = dy;
 	if (offset) {
-		XFillPolygon(XtDisplay(w), XtWindow(w),
-			     w->oct.borderGC,
-			   triangleList[side], 3, Convex, CoordModePrevious);
-		XDrawLines(XtDisplay(w), XtWindow(w),
-			   w->oct.faceGC[w->oct.octaLoc[face][position].face],
-			   triangleList[side], 4, CoordModePrevious);
+		borderGC = w->oct.faceGC[w->oct.octaLoc[face][position].face];
+		if (w->oct.depth < 2 || w->oct.mono) {
+			faceGC = w->oct.inverseGC;
+		} else {
+			faceGC = w->oct.borderGC;
+		}
 	} else {
-		XFillPolygon(XtDisplay(w), XtWindow(w),
-			  w->oct.faceGC[w->oct.octaLoc[face][position].face],
-			   triangleList[side], 3, Convex, CoordModePrevious);
-		XDrawLines(XtDisplay(w), XtWindow(w),
-		  w->oct.borderGC, triangleList[side], 4, CoordModePrevious);
+		faceGC = w->oct.faceGC[w->oct.octaLoc[face][position].face];
+		borderGC = w->oct.borderGC;
 	}
-	if (w->oct.depth == 1 || w->oct.mono) {
+	XFillPolygon(XtDisplay(w), XtWindow(w), faceGC,
+		     triangleList[side], 3, Convex, CoordModePrevious);
+	XDrawLines(XtDisplay(w), XtWindow(w), borderGC,
+		   triangleList[side], 4, CoordModePrevious);
+	if (w->oct.depth < 2 || w->oct.mono) {
 		int         letterX, letterY;
 		char        buf[2];
 
@@ -1880,16 +1911,21 @@ DrawTriangle(OctWidget w, int face, int position, int offset)
 						  [face][position].face][0]);
 		letterX = dx + letterList[side].x;
 		letterY = dy + letterList[side].y;
-		XDrawString(XtDisplay(w), XtWindow(w), w->oct.inverseGC,
+		if (offset) {
+			borderGC = w->oct.borderGC;
+		} else {
+			borderGC = w->oct.inverseGC;
+		}
+		XDrawString(XtDisplay(w), XtWindow(w), borderGC,
 			    letterX, letterY, buf, 1);
 	}
 	if (w->oct.orient)
 		DrawOrientLine(w, w->oct.octaLoc[face][position].rotation, faceOnView,
-			       side, dx, dy);
+			       side, dx, dy, borderGC);
 }
 
 static void
-DrawOrientLine(OctWidget w, int orient, int face, int side, int dx, int dy)
+DrawOrientLine(OctWidget w, int orient, int face, int side, int dx, int dy, GC borderGC)
 {
 	int         x1 = 0, x2 = 0, y1 = 0, y2 = 0;
 	int         temp1 = w->oct.octaLength + 1;
@@ -1900,7 +1936,7 @@ DrawOrientLine(OctWidget w, int orient, int face, int side, int dx, int dy)
 	switch ((side == face) ? orient : (orient + MAXORIENT / 2) % MAXORIENT) {
 		case 0:
 			x2 = x1 = dx;
-			y1 = dy + temp1;
+			y1 = dy + temp1 - 2;
 			y2 = y1 - w->oct.orientLineLength;
 			break;
 		case 1:
@@ -1916,7 +1952,7 @@ DrawOrientLine(OctWidget w, int orient, int face, int side, int dx, int dy)
 			y2 = y1 - temp3;
 			break;
 		case 3:
-			x1 = dx - temp1;
+			x1 = dx - temp1 + 2;
 			x2 = x1 + w->oct.orientLineLength;
 			y2 = y1 = dy;
 			break;
@@ -1934,7 +1970,7 @@ DrawOrientLine(OctWidget w, int orient, int face, int side, int dx, int dy)
 			break;
 		case 6:
 			x2 = x1 = dx;
-			y1 = dy - temp1;
+			y1 = dy - temp1 + 2;
 			y2 = y1 + w->oct.orientLineLength;
 			break;
 		case 7:
@@ -1950,7 +1986,7 @@ DrawOrientLine(OctWidget w, int orient, int face, int side, int dx, int dy)
 			y2 = y1 + temp3;
 			break;
 		case 9:
-			x1 = dx + temp1;
+			x1 = dx + temp1 - 2;
 			x2 = x1 - w->oct.orientLineLength;
 			y2 = y1 = dy;
 			break;
@@ -1969,7 +2005,7 @@ DrawOrientLine(OctWidget w, int orient, int face, int side, int dx, int dy)
 		default:
 			(void) printf("DrawOrientLine: orient %d\n", orient);
 	}
-	XDrawLine(XtDisplay(w), XtWindow(w), w->oct.inverseGC, x1, y1, x2, y2);
+	XDrawLine(XtDisplay(w), XtWindow(w), borderGC, x1, y1, x2, y2);
 }
 
 Boolean
@@ -1983,13 +2019,13 @@ CheckSolved(OctWidget w)
 			if (!position) {
 				test.face = w->oct.octaLoc[face][position].face;
 				test.rotation = w->oct.octaLoc[face][position].rotation;
-			} else if (test.face !=		/*MAXSIDES * view + face */
+			} else if (test.face !=		/* MAXSIDES * view + face */
 				   w->oct.octaLoc[face][position].face ||
 				   (w->oct.orient && test.rotation !=
 				    w->oct.octaLoc[face][position].rotation))
-				return FALSE;
+				return False;
 		}
-	return TRUE;
+	return True;
 }
 
 static int
